﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Collections.ObjectModel
Imports System.Drawing.Printing
Imports System.IO
Imports System.ComponentModel
Imports System.Runtime.Serialization.Formatters.Binary

Public Class Form1
   Private moLayer2 As ShapeFileFeatureLayer
   Private moPrinterPage As PagePrinterLayer
   Private moLayer1 As ShapeFileFeatureLayer
   Private moFeatureExtent As RectangleShape
   Private moStyle1 As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Blue, GeoColor.SimpleColors.Black, 6)
   Private moStyle2 As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Red, GeoColor.SimpleColors.Black, 6)
   Private mbStyle1 As Boolean = True
   Private WithEvents moMainMap As CustomWinFormsMap
   Public Enum EnumPagePosition
      TopLeft = 1
      TopRight = 2
      LowerLeft = 3
      LowerRight = 4
   End Enum
   Public Enum EnumCenterType
      Horizontal = 1
      Vertical = 2
   End Enum
   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

      Me.TextBox1.Text =
         "1. Wait for the map to display, it could take a long time." & ControlChars.CrLf &
         "2. Check the checkbox to switch to the Page Layout View." & ControlChars.CrLf &
         "3. Zoom out with the mouse wheel and notice that the Google Maps layer does not fill the map." & ControlChars.CrLf &
         "4. Click the button to change the coordinate system to Spherical Mercator, the native one for Google Maps." & ControlChars.CrLf &
         "5. When the map has had time to redraw it now fills the map properly."

      PageLayout.Visible = False
      moMainMap = New CustomWinFormsMap
      With moMainMap
         .Dock = DockStyle.Fill
         .MapUnit = GeographyUnit.Meter
         .SRID = 26916
      End With
      pnlMaps.Controls.Add(moMainMap)

      CreateDefaultLayout()



   End Sub

   Private Sub AddCenteredTitleLabel()
      Dim oLabelPrinterLayer As LabelPrinterLayer = New LabelPrinterLayer()
      Dim oPageBoundingbox As RectangleShape

      '***** Setup the text and the font..
      With oLabelPrinterLayer
         .Text = "Page Layout"
         .Font = New GeoFont("Courier New", 10, DrawingFontStyles.Bold)
         .TextBrush = New GeoSolidBrush(GeoColor.StandardColors.Black)

         '***** Set the labels position so that is it centered on the page one inch from the top                        
         oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
         Dim oLabelCenter As PointShape = New PointShape()
         oLabelCenter.X = oPageBoundingbox.UpperRightPoint.X - oPageBoundingbox.Width / 2
         oLabelCenter.Y = oPageBoundingbox.UpperLeftPoint.Y - 0.75

         .PrinterWrapMode = PrinterWrapMode.AutoSizeText
         .ResizeMode = PrinterResizeMode.MaintainAspectRatio
         .SetPosition(5, 0.5, oLabelCenter, PrintingUnit.Inch)
      End With
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      oPrinterOverlay.PrinterLayers.Add(Guid.NewGuid.ToString, oLabelPrinterLayer)

   End Sub


   Public Sub CreateDefaultLayout()
      Dim oExtentRectangle As RectangleShape
      Dim moPrinterOverlay As PrinterInteractiveOverLay


      moPrinterOverlay = New PrinterInteractiveOverLay()
      With PageLayout
         .SRID = 26916
         '***** Setup the map unit, you want to always use feet or meters for printer layout maps
         .MapUnit = GeographyUnit.Feet
         '***** Create the default zoom levels for the layout map
         .ZoomLevelSet = New PrinterZoomLevelSet(.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(.MapUnit))

         '***** Set the background color to gray so there is contrast with the white page
         .BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

         '***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
         '***** The interactive overlay allows the layers to be interacted with
         .InteractiveOverlays.Add("PrinterOverlay", moPrinterOverlay)
         .InteractiveOverlays.MoveToBottom("PrinterOverlay")

         '***** Create the PagePrinterLayer which shows the page boundary and 
         '***** is the area the user will arrange all of the other layers on top of.
         moPrinterPage = New PagePrinterLayer()
         With moPrinterPage
            .Open()
            .BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)
            moPrinterOverlay.PrinterLayers.Add("PrinterPage", moPrinterPage)
            '***** Get the page's extent, scale it up by 10%
            oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
            '***** Set oExtentRectangle as the default current extent
            .PageSize = PrinterPageSize.AnsiC
         End With
         .CurrentExtent = oExtentRectangle

         '***** Set the minimum scale of the map to the last zoom level
         .MinimumScale = .ZoomLevelSet.ZoomLevel20.Scale

         '***** Add a container for the map layers
         AddMapLayerContainer(mGetPageLayoutMap)

         '***** Add a layer to the page layout
         AddMapLayers(mGetPageLayoutMap)

         '***** Add a scale bar
         CreateDefaultScaleBar(mGetPageLayoutMap)

         '***** A refresh is done in the form's Shown event
         'AddLegend()
         '''''AddCenteredTitleLabel()
      End With
   End Sub
   Public Sub AddLegend()
      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()

      ' Create a legend item for the title.
      Dim title As New LegendItem()
      title.TextStyle = New TextStyle("Map Legend",
                                      New GeoFont("Courier New", 10, DrawingFontStyles.Bold),
                                      New GeoSolidBrush(GeoColor.SimpleColors.Black))
      title.Width = 200
      ' Create a legend item for the state borders.  This example uses a modified LineStyle.
      Dim legendItem1 As New LegendItem()
      Dim stateBorderStyle As LineStyle = LineStyles.MajorRoad3

      legendItem1.ImageStyle = stateBorderStyle
      legendItem1.TextStyle = New TextStyle("Roads and Rivers", New GeoFont("Courier New", 8), New GeoSolidBrush(GeoColor.SimpleColors.Black))
      legendItem1.Width = 200
      'legendItem1.Height = 12


      Dim legendItem2 As New LegendItem()
      Dim oColor As New GeoColor(100, GeoColor.GeographicColors.Sand)
      legendItem2.ImageStyle = AreaStyles.CreateSimpleAreaStyle(oColor, GeoColor.SimpleColors.Blue, 2)
      legendItem2.TextStyle = New TextStyle("Suburbs", New GeoFont("Courier New", 8), New GeoSolidBrush(GeoColor.SimpleColors.Black))
      legendItem2.Width = 200

      ' Create A legend item for the airports.  This example uses a .png file.
      Dim legendItem3 As New LegendItem()
      'legendItem3.ImageStyle = new PointStyle(new GeoImage(@"..\..\Resources\airport_small_size3.png"))
      legendItem3.TextStyle = New TextStyle("Airports", New GeoFont("Courier New", 8), New GeoSolidBrush(GeoColor.SimpleColors.Black))
      legendItem3.Width = 200

      ' Create the LegendAdornmentLayer and add the LegendItems.
      Dim legendLayer As New LegendPrinterLayer()
      'legendLayer.BackgroundMask = AreaStyles.CreateLinearGradientStyle(New GeoColor(255, 255, 255, 255), New GeoColor(255, 230, 230, 230), 90, GeoColor.SimpleColors.Black)
      legendLayer.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.White, GeoColor.SimpleColors.Black, 1)

      Dim dHeight As Single = 110
      Dim dWidth As Single = 200
      Dim dPix As Double = CreateGraphics.DpiX

      legendLayer.Height = dHeight
      legendLayer.Width = dWidth

      legendLayer.ResizeMode = PrinterResizeMode.Resizable

      legendLayer.LegendItems.Add(legendItem1)
      legendLayer.LegendItems.Add(legendItem2)
      legendLayer.LegendItems.Add(legendItem3)


      legendLayer.Title = title

      moPrinterOverlay = mGetPrinterOverlay()
      moPrinterOverlay.PrinterLayers.Add("Legend", legendLayer)

      'legendLayer.SetPosition((dWidth / 2) / dPix, (dHeight / 2) / dPix, New PointShape(-2, 3.5), PrintingUnit.Inch)
      legendLayer.SetPosition((dWidth / 1) / dPix, (dHeight / 1) / dPix, New PointShape(-2, 3.5), PrintingUnit.Inch)


   End Sub
   Private moGoogleMapsLayer As CustomGoogleLayer
   Public Sub AddMapLayers(moPageLayoutMap As MapPrinterLayer)
      Dim sPath As String
      moPageLayoutMap.MapUnit = GeographyUnit.Meter
      Dim oSavedStyles As New StyleCollection

      Dim oMainMapOverlay As New LayerOverlay


      moGoogleMapsLayer = New CustomGoogleLayer(PageLayout)
      With moGoogleMapsLayer
         .SetCoordinateSystem()
         .Name = "google"
         .MapType = GoogleMapsMapType.Hybrid
         moPageLayoutMap.Layers.Add(moGoogleMapsLayer)
      End With

      '***** Add layer to 'Main' map
      oMainMapOverlay.Layers.Add("google", moGoogleMapsLayer)

      Dim oAreaStyle As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Blue, 1)
      oAreaStyle.Name = "Stand"
      oSavedStyles.Add(oAreaStyle)

      oAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Red, 1)
      oAreaStyle.Name = "Tract"
      oSavedStyles.Add(oAreaStyle)

      StyleCollection.SerializeStyleCollection(oSavedStyles)


      sPath = "..\..\Data\Crash 2.shp"
      moLayer1 = New ShapeFileFeatureLayer(sPath, ShapeFileReadWriteMode.ReadOnly)
      moPageLayoutMap.Layers.Add(moLayer1)
      Dim oColor As New GeoColor(100, GeoColor.GeographicColors.Sand)
      moLayer1.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(oColor, GeoColor.SimpleColors.Blue, 4)
      moLayer1.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

      oMainMapOverlay.Layers.Add(moLayer1)


      sPath = "..\..\Data\Layer1.shp"
      moLayer2 = New ShapeFileFeatureLayer(sPath, ShapeFileReadWriteMode.ReadWrite)
      moPageLayoutMap.Layers.Add(moLayer2)
      moLayer2.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Grass1
      moLayer2.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

      oMainMapOverlay.Layers.Add(moLayer2)

      moLayer1.Open()
      Dim oExtent As RectangleShape = moLayer1.GetBoundingBox

      moMainMap.Overlays.Add(oMainMapOverlay)
      moMainMap.CurrentExtent = oExtent

      moMainMap.Refresh()

      oExtent = moMainMap.CurrentExtent

      moPageLayoutMap.Open()
      Dim oMapPageExtent As RectangleShape = moPageLayoutMap.GetPosition()


      Dim dCurrentZoom As Double = 100
      PageLayout.CurrentScale = 12 * 96 / (dCurrentZoom / 100)

      moPageLayoutMap.MapExtent = PageLayoutExtentFromMainMapExtent()

   End Sub
   Private Sub mSwitchToSphericalMercator()
      Dim oProj4 As New ManagedProj4Projection

      oProj4.InternalProjectionParametersString = ManagedProj4Projection.GetEsriParametersString(26916)
      oProj4.ExternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3857)

      oProj4.Open()
      moLayer1.FeatureSource.Projection = oProj4
      moLayer2.FeatureSource.Projection = oProj4

      PageLayout.SRID = 3857
      moMainMap.SRID = 3857

      moGoogleMapsLayer.SetCoordinateSystem()

      moLayer1.Open()
      moMainMap.CurrentExtent = moLayer1.GetBoundingBox
      mGetPageLayoutMap.MapExtent = PageLayoutExtentFromMainMapExtent()

      If moMainMap.Visible Then
         moMainMap.Refresh()
      Else
         btnRefresh_Click(Nothing, Nothing)
      End If

   End Sub
   Private Function PageLayoutExtentFromMainMapExtent() As RectangleShape
      Dim oMapExtent As RectangleShape = moMainMap.CurrentExtent
      Dim oPageLayoutExtent As RectangleShape


      Dim oCenterPoint As PointShape = oMapExtent.GetCenterPoint

      Dim oLayoutMap As MapPrinterLayer = mGetPageLayoutMap()
      Dim oPageRect As RectangleShape = oLayoutMap.GetPosition

      Dim dXRatio As Double = oPageRect.Width / moMainMap.Width
      Dim dYRatio As Double = oPageRect.Height / moMainMap.Height

      Dim dNewMinX As Double = oCenterPoint.X - (oMapExtent.Width * dXRatio) / 2
      Dim dNewMaxX As Double = oCenterPoint.X + (oMapExtent.Width * dXRatio) / 2


      Dim dNewMinY As Double = oCenterPoint.Y - (oMapExtent.Height * dYRatio) / 2
      Dim dNewMaxY As Double = oCenterPoint.Y + (oMapExtent.Height * dYRatio) / 2

      oPageLayoutExtent = New RectangleShape(dNewMinX, dNewMaxY, dNewMaxX, dNewMinY)

      Return oPageLayoutExtent
   End Function
   Public Sub AddMapLayerContainer(moPageLayoutMap As MapPrinterLayer)
      Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)

      moPageLayoutMap = New CustomPageLayoutMap()

      moPageLayoutMap.MapUnit = GeographyUnit.Feet

      '***** Create the MapPrinterLayer and set the position
      moPageLayoutMap.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)

      '***** Set the maps position slightly below the pages center and 7.5 inches wide and 9 inches tall
      oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
      moPageLayoutMap.SetPosition(16, 21, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y, PrintingUnit.Inch)

      '***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      moPrinterOverlay.PrinterLayers.Add("MapPrinterLayer", moPageLayoutMap)
   End Sub
   Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
      Dim moPrinterPage As PagePrinterLayer = mGetPrinterPage()
      Return moPrinterPage.GetPosition(unit)
   End Function
   Public Sub CreateDefaultScaleBar(moPageLayoutMap As MapPrinterLayer)
      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      If moPrinterOverlay.PrinterLayers.Contains("ScaleBar") Then
         moPrinterOverlay.PrinterLayers.Remove("ScaleBar")
      End If

      '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
      Dim moScaleBarLayer As ScaleBarPrinterLayer
      moScaleBarLayer = New ScaleBarPrinterLayer(moPageLayoutMap)

      With moScaleBarLayer
         .MapUnit = moPageLayoutMap.MapUnit
         .AlternateBarBrush = New GeoSolidBrush(GeoColor.SimpleColors.Black)
         .TextColor = GeoColor.SimpleColors.Black
         .Font = New GeoFont("Courier New", 10)
         .BarBrush = New GeoSolidBrush(GeoColor.SimpleColors.BrightOrange)
         .UnitFamily = UnitSystem.Imperial
         .MaxWidth = 7
      End With

      moScaleBarLayer.Open()
      mSetPosition(2.5, 0.25, moScaleBarLayer, EnumPagePosition.LowerRight, mGetPageLayoutMap)

      '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("ScaleBar", moScaleBarLayer)
   End Sub
   Public Sub mSetPosition(ByVal dWidth As Double, ByVal dHeight As Double, ByVal oPrinterLayer As PrinterLayer, ByVal eInitialPosition As EnumPagePosition, moPageLayoutMap As MapPrinterLayer)
      Dim dCenterPointX As Double
      Dim dCenterPointY As Double
      Dim dDistanceFromEdge As Double = 0.25

      '***** Get the bounding box for the MapPrinterLayer
      moPageLayoutMap.Open()
      Dim oMapBoundingBox As RectangleShape = moPageLayoutMap.GetPosition(PrintingUnit.Inch)
      Select Case eInitialPosition
         Case EnumPagePosition.LowerRight
            dCenterPointX = oMapBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
         Case EnumPagePosition.LowerLeft
            dCenterPointX = oMapBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
         Case EnumPagePosition.TopRight
            dCenterPointX = oMapBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
         Case EnumPagePosition.TopLeft
            dCenterPointX = oMapBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
      End Select

      '***** Now let's try to place this layer in the lower right corner of the page
      oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)
   End Sub

   Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click
      mRefreshMapLayers(mGetPageLayoutMap)
   End Sub

   Private Sub mRefreshMapLayers(moPageLayoutMap As MapPrinterLayer)
      If Not moPageLayoutMap Is Nothing Then
         moPageLayoutMap.IsDrawing = True
         ''moPageLayoutMap.ClearCache()
      End If

      PageLayout.Refresh()

      If Not moPageLayoutMap Is Nothing Then
         moPageLayoutMap.IsDrawing = False
      End If
   End Sub
   Private Sub mCenterElement(ByVal eCenterType As EnumCenterType, ByVal oPrinterLayer As PrinterLayer)
      Dim oCurPosition As RectangleShape = Nothing
      Dim oNewCenterPoint As PointShape

      oPrinterLayer.Open()
      oCurPosition = oPrinterLayer.GetPosition(PrintingUnit.Inch)

      Dim oPageBoundingBox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)


      If eCenterType = EnumCenterType.Horizontal Then
         oNewCenterPoint = New PointShape(0, oCurPosition.GetCenterPoint.Y)
      Else
         oNewCenterPoint = New PointShape(oCurPosition.GetCenterPoint.X, 0)
      End If

      oPrinterLayer.SetPosition(oCurPosition.Width, oCurPosition.Height, oNewCenterPoint, PrintingUnit.Inch)


      mRefreshMapLayers(mGetPageLayoutMap)
   End Sub
   Public Function GetUserFolder() As String
      Dim sBasePath As String
      Dim sApplicationPath As String

      sBasePath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)
      sApplicationPath = Application.CompanyName & "\" & Application.ProductName & "\"

      If Not System.IO.Directory.Exists(sBasePath & "\" & sApplicationPath) Then
         System.IO.Directory.CreateDirectory(sBasePath & "\" & sApplicationPath)
      End If

      Return sBasePath & "\" & sApplicationPath

   End Function
   Private Function mGetPrinterOverlay() As PrinterInteractiveOverLay
      For Each oPrinterOverlay In PageLayout.InteractiveOverlays
         If TypeOf oPrinterOverlay Is PrinterInteractiveOverLay Then
            Return CType(oPrinterOverlay, PrinterInteractiveOverLay)
         End If
      Next oPrinterOverlay
      Return Nothing
   End Function
   Private Function mGetPageLayoutMap() As CustomPageLayoutMap
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      Dim oPrinterLayer As PrinterLayer
      For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
         If TypeOf oPrinterLayer Is MapPrinterLayer Then
            Return CType(oPrinterLayer, CustomPageLayoutMap)
         End If
      Next oPrinterLayer
      Return Nothing
   End Function
   Private Function mGetPrinterPage() As PagePrinterLayer
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      Dim oPrinterLayer As PrinterLayer
      For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
         If TypeOf oPrinterLayer Is PagePrinterLayer Then
            Return CType(oPrinterLayer, PagePrinterLayer)
         End If
      Next oPrinterLayer
      Return Nothing
   End Function

   Private Sub chkShowLayoutMap_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles chkShowLayoutMap.CheckedChanged
      If chkShowLayoutMap.Checked Then
         moMainMap.Visible = False
         PageLayout.Visible = True
         mGetPageLayoutMap.MapExtent = PageLayoutExtentFromMainMapExtent()
         'PageLayout.SendToBack()
         mRefreshMapLayers(mGetPageLayoutMap)
      Else
         PageLayout.Visible = False
         moMainMap.Visible = True
         'moMainMap.SendToBack()
         moMainMap.Refresh()
      End If
      Dim dScale = ExtentHelper.GetScale(mGetPageLayoutMap.MapExtent, CSng(mGetPageLayoutMap.MapExtent.Width), GeographyUnit.Meter)
   End Sub

   Private Sub btnSwitchCoordSys_Click(sender As System.Object, e As System.EventArgs) Handles btnSwitchCoordSys.Click
      If btnSwitchCoordSys.Text.ToUpper = "SWITCH TO SPHERICAL MERCATOR" Then
         mSwitchToSphericalMercator()
         btnSwitchCoordSys.Text = "Switch To NAD83 Zone 16N"
      Else
         moLayer1.FeatureSource.Projection = Nothing
         moLayer2.FeatureSource.Projection = Nothing
         PageLayout.SRID = 3587
         moMainMap.SRID = 3587
         moGoogleMapsLayer.SetCoordinateSystem()
         moLayer1.Open()
         moMainMap.CurrentExtent = moLayer1.GetBoundingBox
         mGetPageLayoutMap.MapExtent = PageLayoutExtentFromMainMapExtent()
         If PageLayout.Visible Then
            btnRefresh_Click(Nothing, Nothing)
         Else
            moMainMap.Refresh()
         End If
         btnSwitchCoordSys.Text = "Switch To Spherical Mercator"
      End If
   End Sub
End Class
